local Theme = FlexBar2.OO.Mixin 
{
	"SetChecked", -- Check the button in a certain way depending on what type is passed
};


function Theme:Activate()
	self.Frame = CreateFrame("Frame", nil, UIParent);
	self.Frame:SetFrameStrata("LOW");
	self.Frame:SetWidth(10);
	self.Frame:SetHeight(10);
	self.Frame:ClearAllPoints();

	-- Frame settings
	self.Frame:EnableMouse(true);
	self.Frame:SetMovable(true);
	self.Frame:SetToplevel(true);
	self.Frame:SetClampedToScreen(false);

	--dragging handle creation
	self.Frame.Handle = self.Frame:CreateTexture(nil, "ARTWORK");
	self.Frame.Handle:SetTexture("Interface\\Tooltips\\UI-StatusBar-Border");
	self.Frame.Handle:SetAllPoints(self.Frame);

	--Actual button creation (note: not setting a NormalTexture here, an apart function does that which is called later on [at the icon creation part])
	self.Frame.Button = CreateFrame("CheckButton", "FB2_BTN_" .. self.uid, self.Frame, "SecureActionButtonTemplate"); -- Need to set a frame name here for SetBindingClick to work, sigh ...
	self.Frame.Button:SetWidth(36);
	self.Frame.Button:SetHeight(36);
	self.Frame.Button:ClearAllPoints();
	self.Frame.Button:SetPoint("TOPLEFT", self.Frame, "TOPLEFT", 10, -10);
	self.Frame.Button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	self.Frame.Button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	self.Frame.Button:GetHighlightTexture():SetBlendMode("ADD");
	self.Frame.Button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight");
	self.Frame.Button:GetCheckedTexture():SetBlendMode("ADD");

	-- Text1 fontstring creation
	self.Frame.Button.Text1 = self.Frame.Button:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
	self.Frame.Button.Text1:SetJustifyH("LEFT");
	self.Frame.Button.Text1:SetPoint("TOPLEFT", self.Frame.Button, "TOPLEFT");
	self.Frame.Button.Text1:SetPoint("TOPRIGHT", self.Frame.Button, "TOPRIGHT");

	-- Text2 fontstring creation
	self.Frame.Button.Text2 = self.Frame.Button:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
	self.Frame.Button.Text2:SetJustifyH("RIGHT");
	self.Frame.Button.Text2:SetPoint("TOPLEFT", self.Frame.Button, "TOPLEFT");
	self.Frame.Button.Text2:SetPoint("TOPRIGHT", self.Frame.Button, "TOPRIGHT");

	-- Text3 fontstring creation
	self.Frame.Button.Text3 = self.Frame.Button:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
	self.Frame.Button.Text3:SetJustifyH("RIGHT");
	self.Frame.Button.Text3:SetPoint("BOTTOMLEFT", self.Frame.Button, "BOTTOMLEFT");
	self.Frame.Button.Text3:SetPoint("BOTTOMRIGHT", self.Frame.Button, "BOTTOMRIGHT");

	-- Text4 fontstring creation
	self.Frame.Button.Text4 = self.Frame.Button:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
	self.Frame.Button.Text4:SetJustifyH("LEFT");
	self.Frame.Button.Text4:SetPoint("BOTTOMRIGHT", self.Frame.Button, "BOTTOMRIGHT");
	self.Frame.Button.Text4:SetPoint("BOTTOMLEFT", self.Frame.Button, "BOTTOMLEFT");

	-- Text5 fontstring creation
	self.Frame.Button.Text5 = self.Frame.Button:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
	self.Frame.Button.Text5:SetJustifyH("CENTER");
	self.Frame.Button.Text5:SetPoint("CENTER");

	-- Icon creation
	self.Frame.Button.Icon = self.Frame.Button:CreateTexture(nil, "BACKGROUND");
	self.Frame.Button.Icon:SetAllPoints(self.Frame.Button);

	-- Cooldown ticker creation
	self.Frame.Button.Cooldown = CreateFrame("Model", nil, self.Frame.Button, "CooldownFrameTemplate");
	self.LastCooldown = {};

	-- NormalTexture setup
	self.Frame.Button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	self.Frame.Button:GetNormalTexture():SetWidth(64);
	self.Frame.Button:GetNormalTexture():SetHeight(64);
	self.Frame.Button:GetNormalTexture():ClearAllPoints();
	self.Frame.Button:GetNormalTexture():SetPoint("CENTER", 1, -1);

	-- Button settings
	self.Frame.Button:EnableMouse(true);
	self.Frame.Button:SetClampedToScreen(true);

	-- Fallback functions if we get the frame sent instead of the class (happens on rare occasions when using SetScript based triggers)
	self.Frame.GetClass = function() return self; end
	self.Frame.Button.GetClass = function() return self; end
end

function Theme:SetChecked(State, CheckType)
	if(State == true) then
		if(type(self.Check) ~= "table") then
			self.Check = {};
		end
		if(not FlexBar2:ValueIsInTable(self.Check, CheckType)) then
			table.insert(self.Check, CheckType);
			table.sort(self.Check, function(a, b)
				return FlexBar2:GetKeyByValue(FlexBar2.db.profile.ColorPriority, a) < FlexBar2:GetKeyByValue(FlexBar2.db.profile.ColorPriority, b);
			end);
		end
	elseif(State == false) then
		if(type(self.Check) ~= "table") then
			self.Check = {};
		end
		if(FlexBar2:RemoveByValue(self.Check, CheckType)) then
		end
	end
	if(type(self.Check[1]) == "string") then
		self.Frame.Button:SetChecked(true);
		local _, r, g, b = FlexBar2.PaintChips:GetRGBPercent(self.Check[1])
		FlexBar2:Debug("Checking button [" , self.Check[1] , "]" , " (" , r, "," , g , ",", b , ")");
		self.Frame.Button:GetCheckedTexture():SetVertexColor(r, g, b);
	else
		FlexBar2:Debug("Unchecking button [" , CheckType, "]");
		self.Frame.Button:SetChecked(false);
	end
end

FlexBar2:RegisterClass(Theme);
